@@ -3,7 +3,7 @@
  */
 /* Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2011 Olly Betts
  * Copyright 2006,2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -131,27 +131,29 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *  This closes the database and releases all file handles held by the
 	 *  database.
 	 *
-	 *  This is a permanent close of the database: calling reopen() after
-	 *  closing a database will not reopen it, and will raise an exception.
+	 *  This cannot be undone - in particular, calling reopen() after
+	 *  closing a database will not reopen it, but will instead throw a
+	 *  Xapian::DatabaseError exception.
 	 *
-	 *  Calling close() on a database which is already closed has no effect
-	 *  (and doesn't raise an exception).
+	 *  Calling close() again on a database which has already been closed
+	 *  has no effect (and doesn't raise an exception).
 	 *
-	 *  After this call, calls made to methods of the database (other than
-	 *  close() or the destructor), or to objects associated with the
-	 *  database will behave in one of the following ways (but which
-	 *  behaviour happens may vary between releases, and between database
-	 *  backends):
-	 *
-	 *   - raise a Xapian::DatabaseError indicating that the database is
-	 *   closed.
+	 *  After close() has been called, calls to other methods of the
+	 *  database, and to methods of other objects associated with the
+	 *  database, will either:
 	 *
 	 *   - behave exactly as they would have done if the database had not
-	 *   been closed (by using cached data).
+	 *     been closed (this can only happen if all the required data is
+	 *     cached)
 	 *
-	 *  To summarise - you should not rely on the exception being raised,
-	 *  or the normal result being available, but if you do get a result,
-	 *  it will be correct.
+	 *   - raise a Xapian::DatabaseError exception indicating that the
+	 *     database is closed.
+	 *
+	 *  The reason for this behaviour is that otherwise we'd have to check
+	 *  that the database is still open on every method call on every
+	 *  object associated with a Database, when in many cases they are
+	 *  working on data which has already been loaded and so they are able
+	 *  to just behave correctly.
 	 */
 	virtual void close();
 
@@ -171,7 +173,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	/** Corresponding end iterator to postlist_begin().
 	 */
 	PostingIterator postlist_end(const std::string &) const {
-	    return PostingIterator(NULL);
+	    return PostingIterator();
 	}
 
 	/** An iterator pointing to the start of the termlist
@@ -182,7 +184,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	/** Corresponding end iterator to termlist_begin().
 	 */
 	TermIterator termlist_end(Xapian::docid) const {
-	    return TermIterator(NULL);
+	    return TermIterator();
 	}
 
 	/** Does this database have any positional information? */
@@ -196,7 +198,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	/** Corresponding end iterator to positionlist_begin().
 	 */
 	PositionIterator positionlist_end(Xapian::docid, const std::string &) const {
-	    return PositionIterator(NULL);
+	    return PositionIterator();
 	}
 
 	/** An iterator which runs across all terms in the database.
@@ -206,7 +208,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	/** Corresponding end iterator to allterms_begin().
 	 */
 	TermIterator allterms_end() const {
-	    return TermIterator(NULL);
+	    return TermIterator();
 	}
 
 	/** An iterator which runs across all terms with a given prefix.
@@ -225,7 +227,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	/** Corresponding end iterator to allterms_begin(prefix).
 	 */
 	TermIterator allterms_end(const std::string &) const {
-	    return TermIterator(NULL);
+	    return TermIterator();
 	}
 
 	/// Get the number of documents in the database.
@@ -264,12 +266,12 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *  This is the number of documents which have a (non-empty) value
 	 *  stored in the slot.
 	 *
-	 *  @param valno The value slot to examine.
+	 *  @param slot The value slot to examine.
 	 *
 	 *  @exception UnimplementedError The frequency of the value isn't
 	 *  available for this database type.
 	 */
-	Xapian::doccount get_value_freq(Xapian::valueno valno) const;
+	Xapian::doccount get_value_freq(Xapian::valueno slot) const;
 
 	/** Get a lower bound on the values stored in the given value slot.
 	 *
@@ -279,21 +281,21 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *  If the lower bound isn't available for the given database type,
 	 *  this will return the lowest possible bound - the empty string.
 	 *
-	 *  @param valno The value slot to examine.
+	 *  @param slot The value slot to examine.
 	 */
-	std::string get_value_lower_bound(Xapian::valueno valno) const;
+	std::string get_value_lower_bound(Xapian::valueno slot) const;
 
 	/** Get an upper bound on the values stored in the given value slot.
 	 *
 	 *  If there are no values stored in the given value slot, this will
 	 *  return an empty string.
 	 *
-	 *  @param valno The value slot to examine.
+	 *  @param slot The value slot to examine.
 	 *
 	 *  @exception UnimplementedError The upper bound of the values isn't
 	 *  available for this database type.
 	 */
-	std::string get_value_upper_bound(Xapian::valueno valno) const;
+	std::string get_value_upper_bound(Xapian::valueno slot) const;
 
 	/** Get a lower bound on the length of a document in this DB.
 	 *
@@ -335,6 +337,9 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 	 *
 	 *  @exception Xapian::DocNotFoundError      The document specified
 	 *		could not be found in the database.
+	 *
+	 *  @exception Xapian::InvalidArgumentError  did was 0, which is not
+	 *		a valid document id.
 	 */
 	Xapian::Document get_document(Xapian::docid did) const;
 
@@ -360,7 +365,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 
 	/// Corresponding end iterator to spellings_begin().
 	Xapian::TermIterator spellings_end() const {
-	    return Xapian::TermIterator(NULL);
+	    return Xapian::TermIterator();
 	}
 
 	/** An iterator which returns all the synonyms for a given term.
@@ -371,7 +376,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 
 	/// Corresponding end iterator to synonyms_begin(term).
 	Xapian::TermIterator synonyms_end(const std::string &) const {
-	    return Xapian::TermIterator(NULL);
+	    return Xapian::TermIterator();
 	}
 
 	/** An iterator which returns all terms which have synonyms.
@@ -383,7 +388,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 
 	/// Corresponding end iterator to synonym_keys_begin(prefix).
 	Xapian::TermIterator synonym_keys_end(const std::string & = std::string()) const {
-	    return Xapian::TermIterator(NULL);
+	    return Xapian::TermIterator();
 	}
 
 	/** Get the user-specified metadata associated with a given key.
@@ -434,7 +439,7 @@ class XAPIAN_VISIBILITY_DEFAULT Database {
 
 	/// Corresponding end iterator to metadata_keys_begin().
 	Xapian::TermIterator metadata_keys_end(const std::string & = std::string()) const {
-	    return Xapian::TermIterator(NULL);
+	    return Xapian::TermIterator();
 	}
 
 	/** Get a UUID for the database.
