@@ -668,7 +668,7 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 	 *	   should be 0.
 	 *
 	 *  @exception Xapian::InvalidArgumentError will be thrown if an
-	 *  initialised Database object is supplied.
+	 *  empty Database object is supplied.
 	 */
 	explicit Enquire(const Database &database, ErrorHandler * errorhandler_ = 0);
 
@@ -1100,7 +1100,7 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 
 	/** End iterator corresponding to get_matching_terms_begin() */
 	TermIterator get_matching_terms_end(Xapian::docid /*did*/) const {
-	    return TermIterator(NULL);
+	    return TermIterator();
 	}
 
 	/** Get terms which match a given document, by match set item.
@@ -1129,7 +1129,7 @@ class XAPIAN_VISIBILITY_DEFAULT Enquire {
 
 	/** End iterator corresponding to get_matching_terms_begin() */
 	TermIterator get_matching_terms_end(const MSetIterator &/*it*/) const {
-	    return TermIterator(NULL);
+	    return TermIterator();
 	}
 
 	/// Return a string describing this object.
