@@ -1,7 +1,8 @@
 /** \file  queryparser.h
  *  \brief parsing a user query string to build a Xapian::Query object
  */
-/* Copyright (C) 2005,2006,2007,2008,2009,2010 Olly Betts
+/* Copyright (C) 2005,2006,2007,2008,2009,2010,2011 Olly Betts
+ * Copyright (C) 2010 Adam Sjøgren
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -57,11 +58,12 @@ class XAPIAN_VISIBILITY_DEFAULT SimpleStopper : public Stopper {
     SimpleStopper() { }
 
     /// Initialise from a pair of iterators.
-#ifndef __SUNPRO_CC
+#if ! defined __SUNPRO_CC || __SUNPRO_CC - 0 >= 0x580
     template <class Iterator>
     SimpleStopper(Iterator begin, Iterator end) : stop_words(begin, end) { }
 #else
-    // Sun's C++ doesn't cope with the Iterator pointing to const char *.
+    // Older versions of Sun's C++ compiler don't cope with the Iterator
+    // pointing to const char *.
     template <class Iterator>
     SimpleStopper(Iterator begin, Iterator end) {
 	while (begin != end) stop_words.insert(*begin++);
@@ -109,22 +111,22 @@ class XAPIAN_VISIBILITY_DEFAULT StringValueRangeProcessor : public ValueRangePro
   public:
     /** Constructor.
      *
-     *  @param valno_	The value number to return from operator().
+     *  @param slot_	The value number to return from operator().
      */
-    StringValueRangeProcessor(Xapian::valueno valno_)
-	: valno(valno_), str() { }
+    StringValueRangeProcessor(Xapian::valueno slot_)
+	: valno(slot_), str() { }
 
     /** Constructor.
      *
-     *  @param valno_	The value number to return from operator().
+     *  @param slot_	The value number to return from operator().
      *  @param str_     A string to look for to recognise values as belonging
      *                  to this range.
      *  @param prefix_	Flag specifying whether to check for str_ as a prefix
      *			or a suffix.
      */
-    StringValueRangeProcessor(Xapian::valueno valno_, const std::string &str_,
+    StringValueRangeProcessor(Xapian::valueno slot_, const std::string &str_,
 			      bool prefix_ = true)
-	: valno(valno_), prefix(prefix_), str(str_) { }
+	: valno(slot_), prefix(prefix_), str(str_) { }
 
     /** Check for a valid range of this type.
      *
@@ -148,7 +150,7 @@ class XAPIAN_VISIBILITY_DEFAULT DateValueRangeProcessor : public StringValueRang
   public:
     /** Constructor.
      *
-     *  @param valno_	    The value number to return from operator().
+     *  @param slot_	    The value number to return from operator().
      *  @param prefer_mdy_  Should ambiguous dates be interpreted as
      *			    month/day/year rather than day/month/year?
      *			    (default: false)
@@ -156,14 +158,14 @@ class XAPIAN_VISIBILITY_DEFAULT DateValueRangeProcessor : public StringValueRang
      *			    years (default: 1970, so 1/1/69 is 2069 while
      *			    1/1/70 is 1970).
      */
-    DateValueRangeProcessor(Xapian::valueno valno_, bool prefer_mdy_ = false,
+    DateValueRangeProcessor(Xapian::valueno slot_, bool prefer_mdy_ = false,
 			    int epoch_year_ = 1970)
-	: StringValueRangeProcessor(valno_),
+	: StringValueRangeProcessor(slot_),
 	  prefer_mdy(prefer_mdy_), epoch_year(epoch_year_) { }
 
     /** Constructor.
      *
-     *  @param valno_	    The value number to return from operator().
+     *  @param slot_	    The value number to return from operator().
      *
      *  @param str_     A string to look for to recognise values as belonging
      *                  to this date range.
@@ -198,10 +200,10 @@ class XAPIAN_VISIBILITY_DEFAULT DateValueRangeProcessor : public StringValueRang
      *  processor has been added to the queryparser, the queryparser will
      *  accept "created:1/1/2000..31/12/2001".
      */
-    DateValueRangeProcessor(Xapian::valueno valno_, const std::string &str_,
+    DateValueRangeProcessor(Xapian::valueno slot_, const std::string &str_,
 			    bool prefix_ = true,
 			    bool prefer_mdy_ = false, int epoch_year_ = 1970)
-	: StringValueRangeProcessor(valno_, str_, prefix_),
+	: StringValueRangeProcessor(slot_, str_, prefix_),
 	  prefer_mdy(prefer_mdy_), epoch_year(epoch_year_) { }
 
     /** Check for a valid range of this type.
@@ -224,14 +226,14 @@ class XAPIAN_VISIBILITY_DEFAULT NumberValueRangeProcessor : public StringValueRa
   public:
     /** Constructor.
      *
-     *  @param valno_   The value number to return from operator().
+     *  @param slot_   The value number to return from operator().
      */
-    NumberValueRangeProcessor(Xapian::valueno valno_)
-	: StringValueRangeProcessor(valno_) { }
+    NumberValueRangeProcessor(Xapian::valueno slot_)
+	: StringValueRangeProcessor(slot_) { }
 
     /** Constructor.
      *
-     *  @param valno_   The value number to return from operator().
+     *  @param slot_    The value number to return from operator().
      *
      *  @param str_     A string to look for to recognise values as belonging
      *                  to this numeric range.
@@ -261,9 +263,9 @@ class XAPIAN_VISIBILITY_DEFAULT NumberValueRangeProcessor : public StringValueRa
      *  accept "10..50kg" or "10kg..50kg", but not "10..50" or "10kg..50" as
      *  valid ranges.
      */
-    NumberValueRangeProcessor(Xapian::valueno valno_, const std::string &str_,
+    NumberValueRangeProcessor(Xapian::valueno slot_, const std::string &str_,
 			      bool prefix_ = true)
-	: StringValueRangeProcessor(valno_, str_, prefix_) { }
+	: StringValueRangeProcessor(slot_, str_, prefix_) { }
 
     /** Check for a valid range of this type.
      *
@@ -296,6 +298,10 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 	/// Support AND, OR, etc even if they aren't in ALLCAPS.
 	FLAG_BOOLEAN_ANY_CASE = 8,
 	/** Support right truncation (e.g. Xap*).
+	 *
+	 *  Currently you can't use wildcards with boolean filter prefixes,
+	 *  or in a phrase (either an explicitly quoted one, or one implicitly
+	 *  generated by hyphens or other punctuation).
 	 *
 	 *  NB: You need to tell the QueryParser object which database to
 	 *  expand wildcards from by calling set_database.
@@ -316,6 +322,12 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
 	 *  whitespace, to produce more stable results from interactive
 	 *  searches.
 	 *
+	 *  Currently FLAG_PARTIAL doesn't do anything if the final word
+	 *  in the query has a boolean filter prefix, or if it is in a phrase
+	 *  (either an explicitly quoted one, or one implicitly generated by
+	 *  hyphens or other punctuation).  It also doesn't do anything if
+	 *  if the final word is part of a value range.
+	 *
 	 *  NB: You need to tell the QueryParser object which database to
 	 *  expand wildcards from by calling set_database.
 	 */
@@ -435,6 +447,15 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      */
     void set_database(const Database &db);
 
+    /** Specify the maximum expansion of a wildcard term.
+     *
+     *  Note: you must also set FLAG_WILDCARD for wildcard expansion to happen.
+     *
+     *  @param limit	The maximum number of terms each wildcard in the query
+     *			can expand to, or 0 for no limit (which is the default).
+     */
+    void set_max_wildcard_expansion(Xapian::termcount limit);
+
     /** Parse a query.
      *
      *  @param query_string  A free-text query as entered by a user
@@ -547,30 +568,31 @@ class XAPIAN_VISIBILITY_DEFAULT QueryParser {
      *
      *  @param field   The user visible field name
      *  @param prefix  The term prefix to map this to
-     *  @param exclusive If true, each document can have at most one value of
-     *			 the field, so Xapian should combine multiple values
-     *			 with OP_OR.  If false, each document can have multiple
-     *			 values of the field, so Xapian combine them with
-     *			 OP_AND, as we would with filters with different
-     *			 prefixes. [default: true]
+     *  @param exclusive If true, each document can have at most one term with
+     *			 this prefix, so multiple filters with this prefix
+     *			 should be combined with OP_OR.  If false, each
+     *			 document can have multiple terms with this prefix, so
+     *			 multiple filters should be combined with OP_AND, like
+     *			 happens with filters with different prefixes.
+     *			 [default: true]
      */
     void add_boolean_prefix(const std::string &field, const std::string &prefix,
 			    bool exclusive);
 
-    /* FIXME:1.1.3: Merge two versions into one with optional parameter
+    /* FIXME:1.3: Merge two versions into one with optional parameter
      * "exclusive", default true. */
     void add_boolean_prefix(const std::string &field, const std::string &prefix);
 
     /// Iterate over terms omitted from the query as stopwords.
     TermIterator stoplist_begin() const;
     TermIterator stoplist_end() const {
-	return TermIterator(NULL);
+	return TermIterator();
     }
 
     /// Iterate over unstemmed forms of the given (stemmed) term used in the query.
     TermIterator unstem_begin(const std::string &term) const;
     TermIterator unstem_end(const std::string &) const {
-	return TermIterator(NULL);
+	return TermIterator();
     }
 
     /// Register a ValueRangeProcessor.
