@@ -10,8 +10,9 @@
 
 /* queryparser.lemony: build a Xapian::Query object from a user query string.
  *
- * Copyright (C) 2004,2005,2006,2007,2008,2009,2010 Olly Betts
+ * Copyright (C) 2004,2005,2006,2007,2008,2009,2010,2011 Olly Betts
  * Copyright (C) 2007,2008,2009 Lemur Consulting Ltd
+ * Copyright (C) 2010 Adam Sjøgren
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,12 +36,14 @@
 #include "queryparser_internal.h"
 #include <xapian/error.h>
 #include <xapian/unicode.h>
+#include "str.h"
 #include "stringutils.h"
 
 // Include the list of token values lemon generates.
 #include "queryparser_token.h"
 
 #include <algorithm>
+#include <limits>
 #include <list>
 #include <string>
 
@@ -266,6 +269,10 @@ class State {
     void stoplist_resize(size_t s) {
 	qpi->stoplist.resize(s);
     }
+
+    Xapian::termcount get_max_wildcard_expansion() const {
+	return qpi->max_wildcard_expansion;
+    }
 };
 
 string
@@ -379,11 +386,21 @@ Term::as_wildcarded_query(State * state_) const
 
     const list<string> & prefixes = prefix_info->prefixes;
     list<string>::const_iterator piter;
+    Xapian::termcount expansion_count = 0;
+    Xapian::termcount max = state_->get_max_wildcard_expansion();
     for (piter = prefixes.begin(); piter != prefixes.end(); ++piter) {
 	string root = *piter;
 	root += name;
 	TermIterator t = db.allterms_begin(root);
 	while (t != db.allterms_end(root)) {
+	    if (max != 0 && ++expansion_count > max) {
+		string msg("Wildcard ");
+		msg += unstemmed;
+		msg += "* expands to more than ";
+		msg += str(max);
+		msg += " terms";
+		throw Xapian::QueryParserError(msg);
+	    }
 	    subqs.push_back(Query(*t, 1, pos));
 	    ++t;
 	}
@@ -464,7 +481,7 @@ should_stem(const string & term)
 /** Value representing "ignore this" when returned by check_infix() or
  *  check_infix_digit().
  */
-const unsigned UNICODE_IGNORE(-1);
+const unsigned UNICODE_IGNORE = numeric_limits<unsigned>::max();
 
 inline unsigned check_infix(unsigned ch) {
     if (ch == '\'' || ch == '&' || ch == 0xb7 || ch == 0x5f4 || ch == 0x2027) {
@@ -1299,6 +1316,7 @@ TermGroup::as_group(State *state) const
 {
     const Xapian::Stopper * stopper = state->get_stopper();
     size_t stoplist_size = state->stoplist_size();
+    bool default_op_is_positional = is_positional(state->default_op());
 reprocess:
     Query::op default_op = state->default_op();
     vector<Query> subqs;
@@ -1318,6 +1336,8 @@ reprocess:
 		if (stopper && (*stopper)((*i)->name)) {
 		    state->add_to_stoplist(*i);
 		} else {
+		    if (default_op_is_positional)
+			(*i)->need_positions();
 		    subqs.push_back((*i)->get_query_with_auto_synonyms());
 		}
 		begin = ++i;
@@ -1345,6 +1365,8 @@ reprocess:
 		if (stopper && (*stopper)((*i)->name)) {
 		    state->add_to_stoplist(*i);
 		} else {
+		    if (default_op_is_positional)
+			(*i)->need_positions();
 		    subqs.push_back((*i)->get_query_with_auto_synonyms());
 		}
 		begin = ++i;
@@ -1357,11 +1379,13 @@ reprocess:
 		if (stopper && (*stopper)((*j)->name)) {
 		    state->add_to_stoplist(*j);
 		} else {
+		    if (default_op_is_positional)
+			(*i)->need_positions();
 		    subqs2.push_back((*j)->get_query());
 		}
 	    }
 	    Query q_original_terms;
-	    if (is_positional(default_op)) {
+	    if (default_op_is_positional) {
 		q_original_terms = Query(default_op,
 					 subqs2.begin(), subqs2.end(),
 					 subqs2.size() + 9);
@@ -1389,6 +1413,8 @@ reprocess:
 	    if (stopper && (*stopper)((*i)->name)) {
 		state->add_to_stoplist(*i);
 	    } else {
+		if (default_op_is_positional)
+		    (*i)->need_positions();
 		subqs.push_back((*i)->get_query_with_auto_synonyms());
 	    }
 	}
@@ -1405,7 +1431,7 @@ reprocess:
 
     Query * q = NULL;
     if (!subqs.empty()) {
-	if (is_positional(default_op)) {
+	if (default_op_is_positional) {
 	    q = new Query(default_op, subqs.begin(), subqs.end(),
 			     subqs.size() + 9);
 	} else {
@@ -1548,7 +1574,7 @@ class Terms {
 	delete B;\
     } while (0)
 
-#line 1552 "queryparser/queryparser_internal.cc"
+#line 1578 "queryparser/queryparser_internal.cc"
 /* Next is all token values, in a form suitable for use by makeheaders.
 ** This section will be null unless lemon is run with the -m switch.
 */
@@ -1992,9 +2018,9 @@ static void yy_destructor(
     case 21: /* KET */
     case 22: /* EMPTY_GROUP_OK */
 {
-#line 1545 "queryparser/queryparser.lemony"
+#line 1571 "queryparser/queryparser.lemony"
 delete (yypminor->yy0);
-#line 1998 "queryparser/queryparser_internal.cc"
+#line 2024 "queryparser/queryparser_internal.cc"
 }
       break;
     case 25: /* expr */
@@ -2004,17 +2030,17 @@ delete (yypminor->yy0);
     case 31: /* stop_term */
     case 32: /* compound_term */
 {
-#line 1620 "queryparser/queryparser.lemony"
+#line 1646 "queryparser/queryparser.lemony"
 delete (yypminor->yy73);
-#line 2010 "queryparser/queryparser_internal.cc"
+#line 2036 "queryparser/queryparser_internal.cc"
 }
       break;
     case 28: /* prob */
     case 30: /* stop_prob */
 {
-#line 1715 "queryparser/queryparser.lemony"
+#line 1741 "queryparser/queryparser.lemony"
 delete (yypminor->yy12);
-#line 2018 "queryparser/queryparser_internal.cc"
+#line 2044 "queryparser/queryparser_internal.cc"
 }
       break;
     case 33: /* phrase */
@@ -2022,16 +2048,16 @@ delete (yypminor->yy12);
     case 36: /* near_expr */
     case 37: /* adj_expr */
 {
-#line 1916 "queryparser/queryparser.lemony"
+#line 1942 "queryparser/queryparser.lemony"
 (yypminor->yy46)->destroy();
-#line 2028 "queryparser/queryparser_internal.cc"
+#line 2054 "queryparser/queryparser_internal.cc"
 }
       break;
     case 35: /* group */
 {
-#line 1950 "queryparser/queryparser.lemony"
+#line 1976 "queryparser/queryparser.lemony"
 (yypminor->yy76)->destroy();
-#line 2035 "queryparser/queryparser_internal.cc"
+#line 2061 "queryparser/queryparser_internal.cc"
 }
       break;
     default:  break;   /* If no destructor action specified: do nothing */
@@ -2300,7 +2326,7 @@ static void yy_reduce(
   **     break;
   */
       case 0: /* query ::= expr */
-#line 1602 "queryparser/queryparser.lemony"
+#line 1628 "queryparser/queryparser.lemony"
 {
     // Save the parsed query in the State structure so we can return it.
     if (yymsp[0].minor.yy73) {
@@ -2310,30 +2336,30 @@ static void yy_reduce(
 	state->query = Query();
     }
 }
-#line 2314 "queryparser/queryparser_internal.cc"
+#line 2340 "queryparser/queryparser_internal.cc"
         break;
       case 1: /* query ::= */
-#line 1612 "queryparser/queryparser.lemony"
+#line 1638 "queryparser/queryparser.lemony"
 {
     // Handle a query string with no terms in.
     state->query = Query();
 }
-#line 2322 "queryparser/queryparser_internal.cc"
+#line 2348 "queryparser/queryparser_internal.cc"
         break;
       case 2: /* expr ::= prob_expr */
       case 9: /* bool_arg ::= expr */ yytestcase(yyruleno==9);
-#line 1623 "queryparser/queryparser.lemony"
+#line 1649 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy73; }
-#line 2328 "queryparser/queryparser_internal.cc"
+#line 2354 "queryparser/queryparser_internal.cc"
         break;
       case 3: /* expr ::= bool_arg AND bool_arg */
-#line 1626 "queryparser/queryparser.lemony"
+#line 1652 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy73, yymsp[-2].minor.yy73, Query::OP_AND, yymsp[0].minor.yy73, "AND");   yy_destructor(yypParser,4,&yymsp[-1].minor);
 }
-#line 2334 "queryparser/queryparser_internal.cc"
+#line 2360 "queryparser/queryparser_internal.cc"
         break;
       case 4: /* expr ::= bool_arg NOT bool_arg */
-#line 1628 "queryparser/queryparser.lemony"
+#line 1654 "queryparser/queryparser.lemony"
 {
     // 'NOT foo' -> '<alldocuments> NOT foo'
     if (!yymsp[-2].minor.yy73 && (state->flags & QueryParser::FLAG_PURE_NOT)) {
@@ -2342,46 +2368,46 @@ static void yy_reduce(
     BOOL_OP_TO_QUERY(yygotominor.yy73, yymsp[-2].minor.yy73, Query::OP_AND_NOT, yymsp[0].minor.yy73, "NOT");
   yy_destructor(yypParser,5,&yymsp[-1].minor);
 }
-#line 2346 "queryparser/queryparser_internal.cc"
+#line 2372 "queryparser/queryparser_internal.cc"
         break;
       case 5: /* expr ::= bool_arg AND NOT bool_arg */
-#line 1637 "queryparser/queryparser.lemony"
+#line 1663 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy73, yymsp[-3].minor.yy73, Query::OP_AND_NOT, yymsp[0].minor.yy73, "AND NOT");   yy_destructor(yypParser,4,&yymsp[-2].minor);
   yy_destructor(yypParser,5,&yymsp[-1].minor);
 }
-#line 2353 "queryparser/queryparser_internal.cc"
+#line 2379 "queryparser/queryparser_internal.cc"
         break;
       case 6: /* expr ::= bool_arg AND HATE_AFTER_AND bool_arg */
-#line 1640 "queryparser/queryparser.lemony"
+#line 1666 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy73, yymsp[-3].minor.yy73, Query::OP_AND_NOT, yymsp[0].minor.yy73, "AND");   yy_destructor(yypParser,4,&yymsp[-2].minor);
   yy_destructor(yypParser,10,&yymsp[-1].minor);
 }
-#line 2360 "queryparser/queryparser_internal.cc"
+#line 2386 "queryparser/queryparser_internal.cc"
         break;
       case 7: /* expr ::= bool_arg OR bool_arg */
-#line 1643 "queryparser/queryparser.lemony"
+#line 1669 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy73, yymsp[-2].minor.yy73, Query::OP_OR, yymsp[0].minor.yy73, "OR");   yy_destructor(yypParser,2,&yymsp[-1].minor);
 }
-#line 2366 "queryparser/queryparser_internal.cc"
+#line 2392 "queryparser/queryparser_internal.cc"
         break;
       case 8: /* expr ::= bool_arg XOR bool_arg */
-#line 1646 "queryparser/queryparser.lemony"
+#line 1672 "queryparser/queryparser.lemony"
 { BOOL_OP_TO_QUERY(yygotominor.yy73, yymsp[-2].minor.yy73, Query::OP_XOR, yymsp[0].minor.yy73, "XOR");   yy_destructor(yypParser,3,&yymsp[-1].minor);
 }
-#line 2372 "queryparser/queryparser_internal.cc"
+#line 2398 "queryparser/queryparser_internal.cc"
         break;
       case 10: /* bool_arg ::= */
-#line 1655 "queryparser/queryparser.lemony"
+#line 1681 "queryparser/queryparser.lemony"
 {
     // Set the argument to NULL, which enables the bool_arg-using rules in
     // expr above to report uses of AND, OR, etc which don't have two
     // arguments.
     yygotominor.yy73 = NULL;
 }
-#line 2382 "queryparser/queryparser_internal.cc"
+#line 2408 "queryparser/queryparser_internal.cc"
         break;
       case 11: /* prob_expr ::= prob */
-#line 1667 "queryparser/queryparser.lemony"
+#line 1693 "queryparser/queryparser.lemony"
 {
     yygotominor.yy73 = yymsp[0].minor.yy12->query;
     yymsp[0].minor.yy12->query = NULL;
@@ -2419,39 +2445,39 @@ static void yy_reduce(
     }
     delete yymsp[0].minor.yy12;
 }
-#line 2423 "queryparser/queryparser_internal.cc"
+#line 2449 "queryparser/queryparser_internal.cc"
         break;
       case 12: /* prob_expr ::= term */
       case 30: /* stop_term ::= compound_term */ yytestcase(yyruleno==30);
       case 32: /* term ::= compound_term */ yytestcase(yyruleno==32);
-#line 1705 "queryparser/queryparser.lemony"
+#line 1731 "queryparser/queryparser.lemony"
 {
     yygotominor.yy73 = yymsp[0].minor.yy73;
 }
-#line 2432 "queryparser/queryparser_internal.cc"
+#line 2458 "queryparser/queryparser_internal.cc"
         break;
       case 13: /* prob ::= RANGE */
-#line 1717 "queryparser/queryparser.lemony"
+#line 1743 "queryparser/queryparser.lemony"
 {
     valueno slot = yymsp[0].minor.yy0->pos;
     const Query & range = yymsp[0].minor.yy0->as_value_range_query();
     yygotominor.yy12 = new ProbQuery;
     yygotominor.yy12->add_filter_range(slot, range);
 }
-#line 2442 "queryparser/queryparser_internal.cc"
+#line 2468 "queryparser/queryparser_internal.cc"
         break;
       case 14: /* prob ::= stop_prob RANGE */
-#line 1724 "queryparser/queryparser.lemony"
+#line 1750 "queryparser/queryparser.lemony"
 {
     valueno slot = yymsp[0].minor.yy0->pos;
     const Query & range = yymsp[0].minor.yy0->as_value_range_query();
     yygotominor.yy12 = yymsp[-1].minor.yy12;
     yygotominor.yy12->append_filter_range(slot, range);
 }
-#line 2452 "queryparser/queryparser_internal.cc"
+#line 2478 "queryparser/queryparser_internal.cc"
         break;
       case 15: /* prob ::= stop_term stop_term */
-#line 1731 "queryparser/queryparser.lemony"
+#line 1757 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = new ProbQuery;
     yygotominor.yy12->query = yymsp[-1].minor.yy73;
@@ -2469,19 +2495,19 @@ static void yy_reduce(
 	}
     }
 }
-#line 2473 "queryparser/queryparser_internal.cc"
+#line 2499 "queryparser/queryparser_internal.cc"
         break;
       case 16: /* prob ::= prob stop_term */
-#line 1749 "queryparser/queryparser.lemony"
+#line 1775 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = yymsp[-1].minor.yy12;
     // If yymsp[0].minor.yy73 is a stopword, there's nothing to do here.
     if (yymsp[0].minor.yy73) add_to_query(yygotominor.yy12->query, state->default_op(), yymsp[0].minor.yy73);
 }
-#line 2482 "queryparser/queryparser_internal.cc"
+#line 2508 "queryparser/queryparser_internal.cc"
         break;
       case 17: /* prob ::= LOVE term */
-#line 1755 "queryparser/queryparser.lemony"
+#line 1781 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = new ProbQuery;
     if (state->default_op() == Query::OP_AND) {
@@ -2491,10 +2517,10 @@ static void yy_reduce(
     }
   yy_destructor(yypParser,8,&yymsp[-1].minor);
 }
-#line 2495 "queryparser/queryparser_internal.cc"
+#line 2521 "queryparser/queryparser_internal.cc"
         break;
       case 18: /* prob ::= stop_prob LOVE term */
-#line 1764 "queryparser/queryparser.lemony"
+#line 1790 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = yymsp[-2].minor.yy12;
     if (state->default_op() == Query::OP_AND) {
@@ -2507,66 +2533,66 @@ static void yy_reduce(
     }
   yy_destructor(yypParser,8,&yymsp[-1].minor);
 }
-#line 2511 "queryparser/queryparser_internal.cc"
+#line 2537 "queryparser/queryparser_internal.cc"
         break;
       case 19: /* prob ::= HATE term */
-#line 1776 "queryparser/queryparser.lemony"
+#line 1802 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = new ProbQuery;
     yygotominor.yy12->hate = yymsp[0].minor.yy73;
   yy_destructor(yypParser,9,&yymsp[-1].minor);
 }
-#line 2520 "queryparser/queryparser_internal.cc"
+#line 2546 "queryparser/queryparser_internal.cc"
         break;
       case 20: /* prob ::= stop_prob HATE term */
-#line 1781 "queryparser/queryparser.lemony"
+#line 1807 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = yymsp[-2].minor.yy12;
     add_to_query(yygotominor.yy12->hate, Query::OP_OR, yymsp[0].minor.yy73);
   yy_destructor(yypParser,9,&yymsp[-1].minor);
 }
-#line 2529 "queryparser/queryparser_internal.cc"
+#line 2555 "queryparser/queryparser_internal.cc"
         break;
       case 21: /* prob ::= HATE BOOLEAN_FILTER */
-#line 1786 "queryparser/queryparser.lemony"
+#line 1812 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = new ProbQuery;
     yygotominor.yy12->hate = new Query(yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
   yy_destructor(yypParser,9,&yymsp[-1].minor);
 }
-#line 2539 "queryparser/queryparser_internal.cc"
+#line 2565 "queryparser/queryparser_internal.cc"
         break;
       case 22: /* prob ::= stop_prob HATE BOOLEAN_FILTER */
-#line 1792 "queryparser/queryparser.lemony"
+#line 1818 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = yymsp[-2].minor.yy12;
     add_to_query(yygotominor.yy12->hate, Query::OP_OR, yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
   yy_destructor(yypParser,9,&yymsp[-1].minor);
 }
-#line 2549 "queryparser/queryparser_internal.cc"
+#line 2575 "queryparser/queryparser_internal.cc"
         break;
       case 23: /* prob ::= BOOLEAN_FILTER */
-#line 1798 "queryparser/queryparser.lemony"
+#line 1824 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = new ProbQuery;
     yygotominor.yy12->add_filter(yymsp[0].minor.yy0->get_filter_group_id(), yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
 }
-#line 2558 "queryparser/queryparser_internal.cc"
+#line 2584 "queryparser/queryparser_internal.cc"
         break;
       case 24: /* prob ::= stop_prob BOOLEAN_FILTER */
-#line 1804 "queryparser/queryparser.lemony"
+#line 1830 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = yymsp[-1].minor.yy12;
     yygotominor.yy12->append_filter(yymsp[0].minor.yy0->get_filter_group_id(), yymsp[0].minor.yy0->get_query());
     delete yymsp[0].minor.yy0;
 }
-#line 2567 "queryparser/queryparser_internal.cc"
+#line 2593 "queryparser/queryparser_internal.cc"
         break;
       case 25: /* prob ::= LOVE BOOLEAN_FILTER */
-#line 1810 "queryparser/queryparser.lemony"
+#line 1836 "queryparser/queryparser.lemony"
 {
     // LOVE BOOLEAN_FILTER(yymsp[0].minor.yy0) is just the same as BOOLEAN_FILTER
     yygotominor.yy12 = new ProbQuery;
@@ -2574,10 +2600,10 @@ static void yy_reduce(
     delete yymsp[0].minor.yy0;
   yy_destructor(yypParser,8,&yymsp[-1].minor);
 }
-#line 2578 "queryparser/queryparser_internal.cc"
+#line 2604 "queryparser/queryparser_internal.cc"
         break;
       case 26: /* prob ::= stop_prob LOVE BOOLEAN_FILTER */
-#line 1817 "queryparser/queryparser.lemony"
+#line 1843 "queryparser/queryparser.lemony"
 {
     // LOVE BOOLEAN_FILTER(yymsp[0].minor.yy0) is just the same as BOOLEAN_FILTER
     yygotominor.yy12 = yymsp[-2].minor.yy12;
@@ -2587,23 +2613,23 @@ static void yy_reduce(
     delete yymsp[0].minor.yy0;
   yy_destructor(yypParser,8,&yymsp[-1].minor);
 }
-#line 2591 "queryparser/queryparser_internal.cc"
+#line 2617 "queryparser/queryparser_internal.cc"
         break;
       case 27: /* stop_prob ::= prob */
-#line 1832 "queryparser/queryparser.lemony"
+#line 1858 "queryparser/queryparser.lemony"
 { yygotominor.yy12 = yymsp[0].minor.yy12; }
-#line 2596 "queryparser/queryparser_internal.cc"
+#line 2622 "queryparser/queryparser_internal.cc"
         break;
       case 28: /* stop_prob ::= stop_term */
-#line 1834 "queryparser/queryparser.lemony"
+#line 1860 "queryparser/queryparser.lemony"
 {
     yygotominor.yy12 = new ProbQuery;
     yygotominor.yy12->query = yymsp[0].minor.yy73;
 }
-#line 2604 "queryparser/queryparser_internal.cc"
+#line 2630 "queryparser/queryparser_internal.cc"
         break;
       case 29: /* stop_term ::= TERM */
-#line 1848 "queryparser/queryparser.lemony"
+#line 1874 "queryparser/queryparser.lemony"
 {
     if (state->is_stopword(yymsp[0].minor.yy0)) {
 	yygotominor.yy73 = NULL;
@@ -2613,124 +2639,124 @@ static void yy_reduce(
     }
     delete yymsp[0].minor.yy0;
 }
-#line 2617 "queryparser/queryparser_internal.cc"
+#line 2643 "queryparser/queryparser_internal.cc"
         break;
       case 31: /* term ::= TERM */
-#line 1867 "queryparser/queryparser.lemony"
+#line 1893 "queryparser/queryparser.lemony"
 {
     yygotominor.yy73 = new Query(yymsp[0].minor.yy0->get_query_with_auto_synonyms());
     delete yymsp[0].minor.yy0;
 }
-#line 2625 "queryparser/queryparser_internal.cc"
+#line 2651 "queryparser/queryparser_internal.cc"
         break;
       case 33: /* compound_term ::= WILD_TERM */
-#line 1884 "queryparser/queryparser.lemony"
+#line 1910 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy0->as_wildcarded_query(state); }
-#line 2630 "queryparser/queryparser_internal.cc"
+#line 2656 "queryparser/queryparser_internal.cc"
         break;
       case 34: /* compound_term ::= PARTIAL_TERM */
-#line 1887 "queryparser/queryparser.lemony"
+#line 1913 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy0->as_partial_query(state); }
-#line 2635 "queryparser/queryparser_internal.cc"
+#line 2661 "queryparser/queryparser_internal.cc"
         break;
       case 35: /* compound_term ::= QUOTE phrase QUOTE */
-#line 1890 "queryparser/queryparser.lemony"
+#line 1916 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[-1].minor.yy46->as_phrase_query();   yy_destructor(yypParser,19,&yymsp[-2].minor);
   yy_destructor(yypParser,19,&yymsp[0].minor);
 }
-#line 2642 "queryparser/queryparser_internal.cc"
+#line 2668 "queryparser/queryparser_internal.cc"
         break;
       case 36: /* compound_term ::= phrased_term */
-#line 1893 "queryparser/queryparser.lemony"
+#line 1919 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy46->as_phrase_query(); }
-#line 2647 "queryparser/queryparser_internal.cc"
+#line 2673 "queryparser/queryparser_internal.cc"
         break;
       case 37: /* compound_term ::= group */
-#line 1896 "queryparser/queryparser.lemony"
+#line 1922 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy76->as_group(state); }
-#line 2652 "queryparser/queryparser_internal.cc"
+#line 2678 "queryparser/queryparser_internal.cc"
         break;
       case 38: /* compound_term ::= near_expr */
-#line 1899 "queryparser/queryparser.lemony"
+#line 1925 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy46->as_near_query(); }
-#line 2657 "queryparser/queryparser_internal.cc"
+#line 2683 "queryparser/queryparser_internal.cc"
         break;
       case 39: /* compound_term ::= adj_expr */
-#line 1902 "queryparser/queryparser.lemony"
+#line 1928 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[0].minor.yy46->as_adj_query(); }
-#line 2662 "queryparser/queryparser_internal.cc"
+#line 2688 "queryparser/queryparser_internal.cc"
         break;
       case 40: /* compound_term ::= BRA expr KET */
-#line 1905 "queryparser/queryparser.lemony"
+#line 1931 "queryparser/queryparser.lemony"
 { yygotominor.yy73 = yymsp[-1].minor.yy73;   yy_destructor(yypParser,20,&yymsp[-2].minor);
   yy_destructor(yypParser,21,&yymsp[0].minor);
 }
-#line 2669 "queryparser/queryparser_internal.cc"
+#line 2695 "queryparser/queryparser_internal.cc"
         break;
       case 41: /* compound_term ::= SYNONYM TERM */
-#line 1907 "queryparser/queryparser.lemony"
+#line 1933 "queryparser/queryparser.lemony"
 {
     yygotominor.yy73 = new Query(yymsp[0].minor.yy0->get_query_with_synonyms());
     delete yymsp[0].minor.yy0;
   yy_destructor(yypParser,11,&yymsp[-1].minor);
 }
-#line 2678 "queryparser/queryparser_internal.cc"
+#line 2704 "queryparser/queryparser_internal.cc"
         break;
       case 42: /* phrase ::= TERM */
-#line 1918 "queryparser/queryparser.lemony"
+#line 1944 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = new Terms;
     yygotominor.yy46->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2686 "queryparser/queryparser_internal.cc"
+#line 2712 "queryparser/queryparser_internal.cc"
         break;
       case 43: /* phrase ::= phrase TERM */
       case 45: /* phrased_term ::= phrased_term PHR_TERM */ yytestcase(yyruleno==45);
-#line 1923 "queryparser/queryparser.lemony"
+#line 1949 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = yymsp[-1].minor.yy46;
     yygotominor.yy46->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2695 "queryparser/queryparser_internal.cc"
+#line 2721 "queryparser/queryparser_internal.cc"
         break;
       case 44: /* phrased_term ::= TERM PHR_TERM */
-#line 1935 "queryparser/queryparser.lemony"
+#line 1961 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = new Terms;
     yygotominor.yy46->add_positional_term(yymsp[-1].minor.yy0);
     yygotominor.yy46->add_positional_term(yymsp[0].minor.yy0);
 }
-#line 2704 "queryparser/queryparser_internal.cc"
+#line 2730 "queryparser/queryparser_internal.cc"
         break;
       case 46: /* group ::= TERM GROUP_TERM */
-#line 1952 "queryparser/queryparser.lemony"
+#line 1978 "queryparser/queryparser.lemony"
 {
     yygotominor.yy76 = new TermGroup;
     yygotominor.yy76->add_term(yymsp[-1].minor.yy0);
     yygotominor.yy76->add_term(yymsp[0].minor.yy0);
 }
-#line 2713 "queryparser/queryparser_internal.cc"
+#line 2739 "queryparser/queryparser_internal.cc"
         break;
       case 47: /* group ::= group GROUP_TERM */
-#line 1958 "queryparser/queryparser.lemony"
+#line 1984 "queryparser/queryparser.lemony"
 {
     yygotominor.yy76 = yymsp[-1].minor.yy76;
     yygotominor.yy76->add_term(yymsp[0].minor.yy0);
 }
-#line 2721 "queryparser/queryparser_internal.cc"
+#line 2747 "queryparser/queryparser_internal.cc"
         break;
       case 48: /* group ::= group EMPTY_GROUP_OK */
-#line 1963 "queryparser/queryparser.lemony"
+#line 1989 "queryparser/queryparser.lemony"
 {
     yygotominor.yy76 = yymsp[-1].minor.yy76;
     yygotominor.yy76->set_empty_ok();
   yy_destructor(yypParser,22,&yymsp[0].minor);
 }
-#line 2730 "queryparser/queryparser_internal.cc"
+#line 2756 "queryparser/queryparser_internal.cc"
         break;
       case 49: /* near_expr ::= TERM NEAR TERM */
       case 51: /* adj_expr ::= TERM ADJ TERM */ yytestcase(yyruleno==51);
-#line 1974 "queryparser/queryparser.lemony"
+#line 2000 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = new Terms;
     yygotominor.yy46->add_positional_term(yymsp[-2].minor.yy0);
@@ -2740,11 +2766,11 @@ static void yy_reduce(
 	delete yymsp[-1].minor.yy0;
     }
 }
-#line 2744 "queryparser/queryparser_internal.cc"
+#line 2770 "queryparser/queryparser_internal.cc"
         break;
       case 50: /* near_expr ::= near_expr NEAR TERM */
       case 52: /* adj_expr ::= adj_expr ADJ TERM */ yytestcase(yyruleno==52);
-#line 1984 "queryparser/queryparser.lemony"
+#line 2010 "queryparser/queryparser.lemony"
 {
     yygotominor.yy46 = yymsp[-2].minor.yy46;
     yygotominor.yy46->add_positional_term(yymsp[0].minor.yy0);
@@ -2753,7 +2779,7 @@ static void yy_reduce(
 	delete yymsp[-1].minor.yy0;
     }
 }
-#line 2757 "queryparser/queryparser_internal.cc"
+#line 2783 "queryparser/queryparser_internal.cc"
         break;
       default:
         break;
@@ -2782,11 +2808,11 @@ static void yy_parse_failed(
   while( !yypParser->yystack.empty() ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will be executed whenever the
   ** parser fails */
-#line 1549 "queryparser/queryparser.lemony"
+#line 1575 "queryparser/queryparser.lemony"
 
     // If we've not already set an error message, set a default one.
     if (!state->error) state->error = "parse error";
-#line 2790 "queryparser/queryparser_internal.cc"
+#line 2816 "queryparser/queryparser_internal.cc"
   ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 #endif /* YYNOERRORRECOVERY */
@@ -2803,10 +2829,10 @@ static void yy_syntax_error(
   (void)yymajor;
   (void)yyminor;
 #define TOKEN (yyminor.yy0)
-#line 1554 "queryparser/queryparser.lemony"
+#line 1580 "queryparser/queryparser.lemony"
 
     yy_parse_failed(yypParser);
-#line 2810 "queryparser/queryparser_internal.cc"
+#line 2836 "queryparser/queryparser_internal.cc"
   ParseARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
